import { Application, Infra, Either, ok } from '@bitloops/bl-boilerplate-core';
import { Traceable } from '@bitloops/bl-boilerplate-infra-telemetry';
import { Inject } from '@nestjs/common';
import { StreamingIntegrationEventBusToken } from '../../../constants';
import { UserChangedEmailDomainEvent } from '../../../domain/events/UserChangedEmailDomainEvent';
import { UserChangedEmailIntegrationEvent } from '../../../contracts/integration-events/UserChangedEmailIntegrationEvent';
export class UserChangedEmailDomainEventToIntegrationEventHandler
  implements Application.IHandleDomainEvent
{
  constructor(
    @Inject(StreamingIntegrationEventBusToken)
    private readonly integrationEventBus: Infra.EventBus.IEventBus
  ) {}
  get event() {
    return UserChangedEmailDomainEvent;
  }
  get boundedContext(): string {
    return 'iam';
  }
  @Traceable({
    operation: 'UserChangedEmailDomainEventToIntegrationEventHandler',
    metrics: {
      name: 'UserChangedEmailDomainEventToIntegrationEventHandler',
      category: 'domainEventHandler',
    },
  })
  public async handle(
    event: UserChangedEmailDomainEvent
  ): Promise<Either<void, never>> {
    const events = UserChangedEmailIntegrationEvent.create(event);
    await this.integrationEventBus.publish(events);
    return ok();
  }
}
