import {
  Domain,
  Infra,
  asyncLocalStorage,
} from '@bitloops/bl-boilerplate-core';
import { UserChangedEmailDomainEvent } from '../../domain/events/UserChangedEmailDomainEvent';
import { IntegrationUserChangedEmailSchemaV1 } from '../../structs/IntegrationUserChangedEmailSchemaV1';
type TIntegrationSchemas = IntegrationUserChangedEmailSchemaV1;
type ToIntegrationDataMapper = (
  event: UserChangedEmailDomainEvent
) => TIntegrationSchemas;
export class UserChangedEmailIntegrationEvent
  implements Infra.EventBus.IntegrationEvent<TIntegrationSchemas>
{
  public static readonly boundedContextId = 'iam';
  static versions = ['v1'];
  static versionMappers: Record<string, ToIntegrationDataMapper> = {
    v1: UserChangedEmailIntegrationEvent.toIntegrationDatav1,
  };
  public metadata: Infra.EventBus.TIntegrationEventMetadata;
  constructor(public payload: TIntegrationSchemas, version: string) {
    this.metadata = {
      boundedContextId: UserChangedEmailIntegrationEvent.boundedContextId,
      createdTimestamp: Date.now(),
      messageId: new Domain.UUIDv4().toString(),
      correlationId: asyncLocalStorage.getStore()?.get('correlationId'),
      context: asyncLocalStorage.getStore()?.get('context'),
      version,
    };
  }
  static create(
    event: UserChangedEmailDomainEvent
  ): UserChangedEmailIntegrationEvent[] {
    return UserChangedEmailIntegrationEvent.versions.map((version) => {
      const mapper = UserChangedEmailIntegrationEvent.versionMappers[version];
      const payload = mapper(event);
      return new UserChangedEmailIntegrationEvent(payload, version);
    });
  }
  static toIntegrationDatav1(
    event: UserChangedEmailDomainEvent
  ): IntegrationUserChangedEmailSchemaV1 {
    const userChangedEmail = {
      userId: event.payload.aggregateId,
      userEmail: event.payload.email,
    };
    return userChangedEmail;
  }
}
