import {
  Application,
  Either,
  Domain,
  fail,
  ok,
} from '@bitloops/bl-boilerplate-core';
import { Traceable } from '@bitloops/bl-boilerplate-infra-telemetry';
import { CreateUserCommand } from '../commands/CreateUserCommand';
import { Inject } from '@nestjs/common';
import { UserWriteRepoPortToken } from '../../constants';
import { UserWriteRepoPort } from '../../ports/UserWriteRepoPort';
import { CompletedTodosVO } from '../../domain/CompletedTodosVO';
import { EmailVO } from '../../domain/EmailVO';
import { UserEntity } from '../../domain/UserEntity';
export type CreateUserCommandHandlerResponse = Either<
  void,
  Application.Repo.Errors.Unexpected
>;
export class CreateUserCommandHandler
  implements Application.ICommandHandler<CreateUserCommand, void>
{
  constructor(
    @Inject(UserWriteRepoPortToken)
    private readonly userRepo: UserWriteRepoPort
  ) {}
  get command() {
    return CreateUserCommand;
  }
  get boundedContext(): string {
    return 'marketing';
  }
  @Traceable({
    operation: 'CreateUserCommandHandler',
    metrics: {
      name: 'CreateUserCommandHandler',
      category: 'commandHandler',
    },
  })
  async execute(
    command: CreateUserCommand
  ): Promise<CreateUserCommandHandlerResponse> {
    const userId = new Domain.UUIDv4(command.userId);
    const completedTodos = CompletedTodosVO.create({ counter: 0 });
    if (!completedTodos.isFail()) {
      const email = EmailVO.create({ email: command.email });
      if (!email.isFail()) {
        const user = UserEntity.create({
          completedTodos: completedTodos.value,
          email: email.value,
          id: userId,
        });
        if (!user.isFail()) {
          const saveResult = await this.userRepo.save(user.value);
          if (saveResult.isFail()) {
            return fail(saveResult.value);
          }
          return ok();
        } else {
          return fail(user.value);
        }
      } else {
        return fail(email.value);
      }
    } else {
      return fail(completedTodos.value);
    }
  }
}
