import { Application, Either, fail, ok } from '@bitloops/bl-boilerplate-core';
import { Traceable } from '@bitloops/bl-boilerplate-infra-telemetry';
import { TodoReadModel } from '../../domain/TodoReadModel';
import { GetAllTodosQuery } from '../queries/GetAllTodosQuery';
import { Inject } from '@nestjs/common';
import { TodoReadRepoPortToken } from '../../constants';
import { TodoReadRepoPort } from '../../ports/TodoReadRepoPort';
export type GetAllTodosQueryHandlerResponse = Either<
  TodoReadModel[],
  Application.Repo.Errors.Unexpected
>;
export class GetAllTodosQueryHandler
  implements Application.IQueryHandler<GetAllTodosQuery, TodoReadModel[]>
{
  constructor(
    @Inject(TodoReadRepoPortToken)
    private readonly todoRepo: TodoReadRepoPort
  ) {}
  get query() {
    return GetAllTodosQuery;
  }
  get boundedContext(): string {
    return 'todo';
  }
  @Traceable({
    operation: 'GetAllTodosQueryHandler',
    metrics: {
      name: 'GetAllTodosQueryHandler',
      category: 'queryHandler',
    },
  })
  async execute(
    query: GetAllTodosQuery
  ): Promise<GetAllTodosQueryHandlerResponse> {
    const todos = await this.todoRepo.getAll();
    if (todos.isFail()) {
      return fail(todos.value);
    }
    if (!todos.value) {
      return ok([]);
    }
    return ok(todos.value);
  }
}
