import {
  Domain,
  Infra,
  asyncLocalStorage,
} from '@bitloops/bl-boilerplate-core';
import { TodoCompletedDomainEvent } from '../../domain/events/TodoCompletedDomainEvent';
import { IntegrationTodoCompletedSchemaV1 } from '../../structs/IntegrationTodoCompletedSchemaV1';
type TIntegrationSchemas = IntegrationTodoCompletedSchemaV1;
type ToIntegrationDataMapper = (
  event: TodoCompletedDomainEvent
) => TIntegrationSchemas;
export class TodoCompletedIntegrationEvent
  implements Infra.EventBus.IntegrationEvent<TIntegrationSchemas>
{
  public static readonly boundedContextId = 'todo';
  static versions = ['v1'];
  static versionMappers: Record<string, ToIntegrationDataMapper> = {
    v1: TodoCompletedIntegrationEvent.toIntegrationDatav1,
  };
  public metadata: Infra.EventBus.TIntegrationEventMetadata;
  constructor(public payload: TIntegrationSchemas, version: string) {
    this.metadata = {
      boundedContextId: TodoCompletedIntegrationEvent.boundedContextId,
      createdTimestamp: Date.now(),
      messageId: new Domain.UUIDv4().toString(),
      correlationId: asyncLocalStorage.getStore()?.get('correlationId'),
      context: asyncLocalStorage.getStore()?.get('context'),
      version,
    };
  }
  static create(
    event: TodoCompletedDomainEvent
  ): TodoCompletedIntegrationEvent[] {
    return TodoCompletedIntegrationEvent.versions.map((version) => {
      const mapper = TodoCompletedIntegrationEvent.versionMappers[version];
      const payload = mapper(event);
      return new TodoCompletedIntegrationEvent(payload, version);
    });
  }
  static toIntegrationDatav1(
    event: TodoCompletedDomainEvent
  ): IntegrationTodoCompletedSchemaV1 {
    const todoCompleted = {
      todoId: event.payload.aggregateId,
      userId: event.payload.userId,
    };
    return todoCompleted;
  }
}
